import warnings

import numpy as np
import torch
import torch.nn as nn

warnings.filterwarnings('ignore')
np.set_printoptions(precision=3)


class VAE:
    def __init__(self, epochs=50, batch_size=10, lr=1e-2):
        self.epochs = epochs
        self.batch_size = batch_size
        self.lr = lr
        self.loss_func = nn.MSELoss()

    def loss_function(self, generated, original, mu, logvar):
        reconstruction_function = nn.MSELoss(size_average=False)
        MSE = reconstruction_function(generated, original)
        KLD_element = mu.pow(2).add_(logvar.exp()).mul_(-1).add_(1).add_(logvar)
        KLD = torch.sum(KLD_element).mul_(-0.5)
        return MSE + KLD

    def pre_train(self, dataLoader, net, optimizer):
        for index, (order, label, data) in enumerate(dataLoader):
            net.train()
            out1, out21, out22, z, out3 = net(data)
            loss = self.loss_function(out3, data, out21, out22)
            loss = loss / data.shape[0]
            optimizer.zero_grad()
            loss.backward()
            optimizer.step()
        return net, optimizer

    def train(self, dataLoader, data_size, net, optimizer):
        train_loss = 0
        hid1, hid2, out, order_list = [], [], [], []
        for index, (order, label, data) in enumerate(dataLoader):
            net.train()
            out1, out21, out22, z, out3 = net(data)
            loss = self.loss_func(out3, data)
            train_loss += np.abs(loss.item()) * data.shape[0]
            loss = self.loss_function(out3, data, out21, out22)
            loss = loss / data.shape[0]
            optimizer.zero_grad()
            loss.backward()
            optimizer.step()
            hid1.append(out1)
            hid2.append(z)
            out.append(out3)
            order_list.append(order)
        return train_loss / data_size, net, optimizer, hid1, hid2, out, order_list

    def test(self, dataLoader, data_size, net):
        test_loss = 0
        hid1, hid2, out, order_list = [], [], [], []
        for index, (order, label, data) in enumerate(dataLoader):
            net.eval()
            out1, out21, out22, z, out3 = net(data)
            loss = self.loss_function(out3, data, out21, out22)
            test_loss += np.abs(loss.item())
            hid1.append(out1)
            hid2.append(z)
            out.append(out3)
            order_list.append(order)
        return test_loss / data_size, hid1, hid2, out, order_list
